/*------------------------------------------------------\
|                                                       |
|  Title   :   Structures and fonctions for AUT         |
|                                                       |
|  Date    :            03.12.96                        |
|                                                       |
|  Author  :        Jacomme Ludovic                     |
|                                                       |
\------------------------------------------------------*/

# ifndef AUT_102_H
# define AUT_102_H

/*------------------------------------------------------\
|                                                       |
|                      Constants                        |
|                                                       |
\------------------------------------------------------*/
/*------------------------------------------------------\
|                                                       |
|                       Hash Key                        |
|                                                       |
\------------------------------------------------------*/

# define AUT_HASH_KEY_EMPTY      (char *)0
# define AUT_HASH_KEY_DELETED    (char *)1

/*------------------------------------------------------\
|                                                       |
|                      Prime Number                     |
|                                                       |
\------------------------------------------------------*/

# define AUT_MAX_PRIME_NUMBER   58

/*------------------------------------------------------\
|                                                       |
|                        Alloc                          |
|                                                       |
\------------------------------------------------------*/

# define AUT_ALLOC_BLOCK  0
# define AUT_ALLOC_HEAP   1

/*------------------------------------------------------\
|                                                       |
|                        Macros                         |
|                                                       |
\------------------------------------------------------*/
/*------------------------------------------------------\
|                                                       |
|                    Allocation Macros                  |
|                                                       |
\------------------------------------------------------*/

# define autallocblock( S )  ( autalloc( ( S ), AUT_ALLOC_BLOCK ) )
# define autallocheap( S )   ( autalloc( ( S ), AUT_ALLOC_HEAP ) )
# define autfreeblock( P )   ( autfree( ( P ), 0 ) )
# define autfreeheap( P, S ) ( autfree( ( P ), ( S ) ) )

/*------------------------------------------------------\
|                                                       |
|                      Resize Macros                    |
|                                                       |
\------------------------------------------------------*/

# define autresizeblock( P, O, N ) ( autresize( ( P ), ( O ), ( N ) ) )

/*------------------------------------------------------\
|                                                       |
|                    Debug Macros                       |
|                                                       |
\------------------------------------------------------*/

# define autbegin() do {                               \
                                                       \
  if ( AUT_DEBUG_ON ) addautdebug(__LINE__,__FILE__); } while (0)

# define autend()   do {                               \
                                                       \
  if ( AUT_DEBUG_ON ) delautdebug(); } while (0)

/*------------------------------------------------------\
|                                                       |
|                         Type                          |
|                                                       |
\------------------------------------------------------*/
/*------------------------------------------------------\
|                                                       |
|                   Aut Debug List                      |
|                                                       |
\------------------------------------------------------*/

  typedef struct autdebug_list
  {
    struct autdebug_list *NEXT;
    int                   LINE;
    char                 *NAME;

  } autdebug_list;

/*------------------------------------------------------\
|                                                       |
|                   Aut Hash Table                      |
|                                                       |
\------------------------------------------------------*/

  typedef struct authelem
  {
    char *KEY;
    long  VALUE;

  } authelem;

  typedef struct authtable
  {
    authelem *TABLE;
    long      TABLE_SIZE;
    long      NUMBER_ELEM;
    long      NUMBER_ADD;
    long      NUMBER_SCAN;
    long      NUMBER_DEL;
    long      NUMBER_STRETCH;
    long    (*FUNC_SIZE)();
    long    (*FUNC_INDEX)();
    long    (*FUNC_KEY)();

  } authtable;

/*------------------------------------------------------\
|                                                       |
|                   Aut Hash Table 2                    |
|                                                       |
\------------------------------------------------------*/

  typedef struct auth2elem
  {
    char *KEY1;
    char *KEY2;
    long  VALUE;

  } auth2elem;

  typedef struct auth2table
  {
    auth2elem *TABLE;
    long       TABLE_SIZE;
    long       NUMBER_ELEM;
    long       NUMBER_ADD;
    long       NUMBER_SCAN;
    long       NUMBER_DEL;
    long       NUMBER_STRETCH;
    long     (*FUNC_SIZE)();
    long     (*FUNC_INDEX)();
    long     (*FUNC_KEY)();

  } auth2table;

/*------------------------------------------------------\
|                                                       |
|                    Aut Graph                          |
|                                                       |
\------------------------------------------------------*/

  typedef struct autnode_list
  {
    struct autnode_list  *NEXT;
    struct autnode_list **PREV;
    char                 *NAME;
    chain_list           *ARC_FROM;
    chain_list           *ARC_TO;
    long                  FLAGS;
    void                 *USER;

  } autnode_list;

  typedef struct autarc_list
  {
    struct autarc_list  *NEXT;
    struct autarc_list **PREV;
    autnode_list        *NODE_FROM;
    autnode_list        *NODE_TO;
    long                 FLAGS;
    void                *USER;

  } autarc_list;

  typedef struct autgraph
  {
    autnode_list  *NODE;
    autarc_list   *ARC;
    authtable     *HASH_NODE;
    auth2table    *HASH_ARC;
    long           NUMBER_NODE;
    long           NUMBER_ARC;
    long           FLAGS;
    void          *USER;

  } autgraph;

/*------------------------------------------------------\
|                                                       |
|                     Global Variables                  |
|                                                       |
\------------------------------------------------------*/

 extern autdebug_list *HEAD_AUTDEBUG;
 extern char           AUT_DEBUG_ON;
 extern long           AUT_HASH_PRIME_NUMBER[ AUT_MAX_PRIME_NUMBER ];
 extern long           AUT_HASH_MAX_SCAN;
 extern long           AUT_HASH_STRETCH_FACTOR;

/*------------------------------------------------------\
|                                                       |
|                        Functions                      |
|                                                       |
\------------------------------------------------------*/
/*------------------------------------------------------\
|                                                       |
|                       Env Functions                   |
|                                                       |
\------------------------------------------------------*/

  extern void   autenv();

/*------------------------------------------------------\
|                                                       |
|                      Exit Functions                   |
|                                                       |
\------------------------------------------------------*/

  extern void   autexit();

/*------------------------------------------------------\
|                                                       |
|                    Alloc Functions                    |
|                                                       |
\------------------------------------------------------*/

  extern char          *autalloc();
  extern authelem      *allocauthelem();
  extern authtable     *allocauthtable();
  extern auth2elem     *allocauth2elem();
  extern auth2table    *allocauth2table();
  extern autdebug_list *allocautdebug();

/*------------------------------------------------------\
|                                                       |
|                    Resize Functions                   |
|                                                       |
\------------------------------------------------------*/

  extern char  *autresize();

/*------------------------------------------------------\
|                                                       |
|                      Free Functions                   |
|                                                       |
\------------------------------------------------------*/

  extern void  autfree();
  extern void  freeauthelem();
  extern void  freeauthtable();
  extern void  freeauth2elem();
  extern void  freeauth2table();
  extern void  freeautdebug();

/*------------------------------------------------------\
|                                                       |
|                     Debug Functions                   |
|                                                       |
\------------------------------------------------------*/

  extern void  autdebug();
  extern void  addautdebug();
  extern void  delautdebug();

/*------------------------------------------------------\
|                                                       |
|                   Get Hash Functions                  |
|                                                       |
\------------------------------------------------------*/

  extern long       getauthsize();
  extern long       getauthkey();
  extern long       getauthindex();

  extern long       getauth2size();
  extern long       getauth2key();
  extern long       getauth2index();

/*------------------------------------------------------\
|                                                       |
|                   Check Hash Functions                |
|                                                       |
\------------------------------------------------------*/

  extern int        checkauthkey();
  extern int        checkauth2key();

/*------------------------------------------------------\
|                                                       |
|                  Set Hash Functions                   |
|                                                       |
\------------------------------------------------------*/

  extern void       setauthfunc();
  extern void       setauth2func();

/*------------------------------------------------------\
|                                                       |
|                  Hash Table Functions                 |
|                                                       |
\------------------------------------------------------*/

  extern authtable *createauthtable();
  extern void       stretchauthtable();
  extern void       destroyauthtable();
  extern void       resetauthtable();
  extern void       viewauthtable();
  extern void       viewauthelem();

  extern auth2table *createauth2table();
  extern void        stretchauth2table();
  extern void        destroyauth2table();
  extern void        resetauth2table();
  extern void        viewauth2table();
  extern void        viewauth2elem();

/*------------------------------------------------------\
|                                                       |
|                  Hash Element Functions               |
|                                                       |
\------------------------------------------------------*/

  extern authelem  *addauthelem();
  extern authelem  *searchauthelem();
  extern int        delauthelem();

  extern auth2elem  *addauth2elem();
  extern auth2elem  *searchauth2elem();
  extern int         delauth2elem();

/*------------------------------------------------------\
|                                                       |
|                  Hash Test Functions                  |
|                                                       |
\------------------------------------------------------*/

  extern int   testauthtable(); 
  extern int   testauth2table(); 

/*------------------------------------------------------\
|                                                       |
|                  Sort Compare Function                |
|                                                       |
\------------------------------------------------------*/

  extern long sortautcompare();

/*------------------------------------------------------\
|                                                       |
|                    Sort Function                      |
|                                                       |
\------------------------------------------------------*/

  extern void sortautarray();

/*------------------------------------------------------\
|                                                       |
|                 Graph Functions                       |
|                                                       |
\------------------------------------------------------*/

  extern autnode_list *searchautgraphnode();
  extern autarc_list  *searchautgrapharc();

  extern autnode_list *addautgraphnode();
  extern autarc_list  *addautgrapharc();

  extern void          delautgraphnode();
  extern void          delautgrapharc();

  extern autgraph     *createautgraph();
  extern void          destroyautgraph();

  extern void          viewautgraph();
  extern autgraph     *loadautgraph();
  extern void          driveautgraph();

# endif
