/****************************************************************************/
/*                                                                          */
/*                      Chaine de CAO & VLSI   Alliance                     */
/*                                                                          */
/*    Produit : TAS Version 5                                               */
/*    Fichier : tas521.h                                                    */
/*                                                                          */
/*    (c) copyright 1991-1995 Laboratoire MASI equipe CAO & VLSI            */
/*    Tous droits reserves                                                  */
/*    Support : e-mail cao-vlsi@masi.ibp.fr                                 */
/*                                                                          */
/*    Auteur(s) : Payam KIANI                                               */
/*                                                                          */
/****************************************************************************/
/* les structures et base de donnees de TAS                                 */
/****************************************************************************/

#ifndef TAS
#define TAS

/* LISTE DES INCLUDE */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <signal.h>

/* LISTE DES DEFINE */

/* erreurs de tas */
#define TAS_NULL           0
#define TAS_ERROR          1
#define TAS_WARNING        2
#define TAS_BLOC_SIZE      (long)1024

/* defines utilises pour des affectations */
#define TAS_NOTIME         ((long) 0x80000000 ) /* pas de TPxx  */
#define TAS_NOFRONT        ((long) 0x80000000 ) /* pas de front */
#define TAS_NORES          ((long) 0x00000000 ) /* pas de res  */
#define TAS_NOS            ((long) 0x00000000 ) /* pas de s     */
#define TAS_NOCAPA         ((long) 0x00000000 ) /* pas de c     */
#define TAS_NOWIDTH        ((long) 0x00000000 ) /* pas de w     */
#define TAS_ELM            ((char) 'E'        ) /* elmore save  */
#define TAS_RCN            ((char) 'R'        ) /* rcn save     */

/* mots clefs des champs USER des locon */
#define TAS_CON_CARAC      ((long) 0x20000001 )
#define TAS_CON_BYPASSIN   ((long) 0x20000002 )
#define TAS_CON_BYPASSOUT  ((long) 0x20000003 )
#define TAS_CON_TAGCONE    ((long) 0x20000004 )
#define TAS_CON_TAGOUT     ((long) 0x20000005 )
#define TAS_CON_TAGUSED    ((long) 0x20000006 )
#define TAS_CON_TAGCAPA    ((long) 0x20000007 )
#define TAS_CON_BLDUP      ((long) 0x20000008 )
#define TAS_CON_BLDDOWN    ((long) 0x20000009 )

/* mots clefs des champs USER des lotrs */
#define TAS_BTCONFL        ((long) 0x20000010 )
#define TAS_UCONFL         ((long) 0x20000020 )
#define TAS_CAPA_PASS      ((long) 0x20000030 )
#define TAS_TRANS_TAGCONE  ((long) 0x20000040 )
#define TAS_TRANS_TAGOUT   ((long) 0x20000050 )
#define TAS_TRANS_TAGCAPA  ((long) 0x20000060 )
#define TAS_TRANS_TAGUSED  ((long) 0x20000070 )
#define TAS_TRANS_USED     ((long) 0x20000080 )
#define TAS_CAPA_USED      ((long) 0x20000090 )
#define TAS_TRANS_PARA     ((long) 0x200000a0 )
#define TAS_TRANS_BLDUP    ((long) 0x200000b0 )
#define TAS_TRANS_BLDDOWN  ((long) 0x200000c0 )
#define TAS_TRANS_INOUT    ((long) 0x200000d0 )

/* mots clefs des champs USER des cones */
#define TAS_SLOPE_MIN      ((long) 0x20000100 )
#define TAS_SLOPE_MAX      ((long) 0x20000200 )
#define TAS_INCONE         ((long) 0x20000300 )
#define TAS_IN_BACK        ((long) 0x20000400 )
#define TAS_SIG_CONE       ((long) 0x20000500 )

/* defines utilises pour masquage pour link */
#define TAS_INCONE_NINOUT  ((long) 0x10000000 )

/* mots clefs des champs USER des incone */
#define TAS_DELAY_MIN      ((long) 0x20001000 )
#define TAS_DELAY_MAX      ((long) 0x20002000 )
#define TAS_IN_CONESYM     ((long) 0x20003000 )

/* mots clefs des champs USER des branch */
#define TAS_RESIST         ((long) 0x20010000 )

/* defines utilises pour masquage pour link */
#define TAS_LINK_BL        ((long) 0x10000000 )
#define TAS_LINK_INOUT     ((long) 0x20000000 )

/* mots clefs des champs USER des link */
#define TAS_LINK_CARAC     ((long) 0x20100000 )
#define TAS_LINK_BLDUP     ((long) 0x20200000 )
#define TAS_LINK_BLDDOWN   ((long) 0x20300000 )

/* marquage des branches contenant des switch */
#define TAS_SWITCH         ((long) 0x10000000 ) /* part of switch         */

/* marquage des branches et des incone */
#define TAS_PATH_INOUT     ((long) 0x01000000 ) /* branches INOUT          */
#define TAS_EXT_IN         ((long) 0x02000000 ) 
#define TAS_EXT_OUT        ((long) 0x04000000 ) 
#define TAS_IN_MEMSYM      ((long) 0x08000000 ) 
#define TAS_IN_NOTMEMSYM   ((long) 0x00100000 ) 

/* defines utilises pour masquage pour cones */
#define TAS_MARQUE         ((long) 0x10000000 )
#define TAS_DEJAEMPILE     ((long) 0x20000000 )
#define TAS_NOPRECH        ((long) 0x40000000 ) /* noeud connecteur  */
#define TAS_CONE_CONNECT   ((long) 0x80000000 ) /* pour typer un cone con*/
#define TAS_LCOMMAND       ((long) 0x01000000 )
#define TAS_PRECHARGE      ((long) 0x02000000 ) /* noeud precharge   */
#define TAS_INTER          ((long) 0x04000000 ) /* noeud precharge   */
#define TAS_ECONE          ((long) 0x08000000 ) /* noeud precharge   */
#define TAS_BYPASSOUT      ((long) 0x00100000 ) /* cone non-important OUT*/
#define TAS_BYPASSIN       ((long) 0x00200000 ) /* cone non-important IN */
#define TAS_CONE_INV       ((long) 0x00400000 ) /* cone inverseur        */
#define TAS_CONE_BUF       ((long) 0x00800000 ) /* cone buffer           */
#define TAS_CONE_MASK      ((long) 0x0000FFFF ) /* initialisation du type*/

/* defines utilises pour figure hierarchique */
 
#define TAS_ALIM           ((char) 'A')         /* signal alimentation       */

/* mots clefs des champs USER de la figure hierarchique */

#define TAS_SIG_LOCON      ((long) 0x40000001 ) /* signal => connecteur      */
#define TAS_LOCON_SIG      ((long) 0x40000002 ) /* connecteur => signal      */
#define TAS_SIG_NEWSIG     ((long) 0x40000003 ) /* signal <=> signal         */
#define TAS_LOCON_NSIG     ((long) 0x40000004 ) /* connecteur => signal      */
#define TAS_DELAY_NNRC     ((long) 0x40000005 ) /* delay rc entre NN         */
#define TAS_LOSIG_BUF      ((long) 0x40000006 ) /* info du .inf              */
#define TAS_SIG_BYPASSIN   ((long) 0x40000007 ) /* info du .inf              */
#define TAS_SIG_BYPASSOUT  ((long) 0x40000008 ) /* info du .inf              */
#define TAS_SIG_INTER      ((long) 0x40000009 ) /* info du .inf              */
#define TAS_SIG_ELM        ((long) 0x4000000a ) /* info du .inf              */
#define TAS_SIG_RCN        ((long) 0x4000000b ) /* info du .inf              */
#define TAS_SIG_RC         ((long) 0x4000000d ) /* info du .inf              */
#define TAS_SIG_NORC       ((long) 0x4000000e ) /* info du .inf              */

/* marquage des noeuds pour les parcours de graphe */

#define TAS_NODE_LOOP      ((long) 0x01000000 ) /* noeud dans une loop       */
#define TAS_NODE_USED      ((long) 0x02000000 ) /* noeud deja parcouru       */
#define TAS_NODE_USEDDUAL  ((long) 0x04000000 ) /* noeud deja parcouru       */
#define TAS_NODE_LINEEXT   ((long) 0x08000000 ) /* noeud deja parcouru       */
#define TAS_LINE_LOOP      ((long) 0x01000000 ) /* lien sur une boucle       */
#define TAS_LINE_EXT       ((long) 0x02000000 ) /* lien sur un chemin ext    */
#define TAS_SIG_EXT        ((long) 0x04000000 ) /* signal sur un chemin ext  */

/* les pseudo-fonctions */
#define TAS_GETCLINK(link)  ((caraclink_list *)getptype(link->USER, \
                              TAS_LINK_CARAC)->DATA)
#define TAS_GETWIDTH(link)  ((long)((caraclink_list *)getptype(link->USER, \
                                     TAS_LINK_CARAC)->DATA)->WIDTH)
#define TAS_GETCAPARA(link) ((TAS_PATH_TYPE == 'M') ? \
                             ((long)((caraclink_list *)getptype(link->USER, \
                                     TAS_LINK_CARAC)->DATA)->CRAM) : (long)0)
#define TAS_GETCAPAEQ(link) ((long)((caraclink_list *)getptype(link->USER, \
                                     TAS_LINK_CARAC)->DATA)->CEQUI)
#define TAS_F1(a,b,r)       ((double)(a*sqrt( b/(b+(r*a)) )))
#define TAS_F2(a,b,r)       ((double)sqrt( b*(b+(r*a)) ))
#define TAS_F3(a,b,i)       ((double) (((i*b)+sqrt(i*b*i*b+(4*a*i))) / (2*a)) )
#define TAS_F4(u,vdd,fin)   ((double)(fin*u/vdd))
#define TAS_F5(yp,yn,qp,qn) ((double)(sqrt((yp*qp)/(yn*qn))))
#define TAS_F6(u,b)         ((double)(0.5*(((1.0+b)*(log(1.0+u))/(1.0-b))\
                              -log(1-u)-2*(log(1+b*u))/(b-b*b))))
#define TAS_TGH(x)          ((double)((exp(x)-exp((-x)))/(exp(x)+exp((-x)))))

/* STRUCTURES DE TAS */

typedef struct delay
        {
         long    TPLH,TPHL;
         long    TPLL,TPHH;
         long    FLH,FHL;
         long    FLL,FHH;
         long    RLH,RHL;
         long    RLL,RHH;
         long    SLH,SHL;
         long    SLL,SHH;
        }
delay_list;

typedef struct front
        {
         long      FUP,FDOWN;
        }
front_list;

typedef struct caraccon
        {
         long      RUPMAX,RDOWNMAX;
         long      RUPMIN,RDOWNMIN;
         long      C;
        }
caraccon_list;

typedef struct caraclink
        {
         long      WIDTH;
         long      CRAM;
         long      CEQUI;
        }
caraclink_list;

typedef struct information /* informations relatives au circuit */
    {
     long lo_t     ; /* temps du chargement de le netlist logique */
     long db_t     ; /* temps du desassemblage                    */
     long el_t     ; /* temps du calcul des temps elementaires    */
     long gr_t     ; /* temps du parcours de graphe               */
     long pr_t     ; /* temps de generation du perfmodule         */
     long nb_trans ; /* nombre de transistors                     */
     long nb_cone  ; /* nombre de cones                           */
     long comp     ; /* complexite du circuit                     */
     long maxdelay ; /* delai max du bloc                         */
     long mindelay ; /* delai min du bloc                         */
     short argc    ; /* nombre d'arguments de la ligne de commande*/
     char **argv   ; /* arguments de la ligne de commande         */
    }
info_list ;

/* declaration de variable */
extern double         RN        , 
                      RP        , 
                      FRONT_CON , 
                      AN        , 
                      BN        , 
                      AP        , 
                      BP        , 
                      RNT       , 
                      RPT       , 
                      VDDmax    , 
                      VTP       , 
                      VTN       , 
                      SEUIL     , 
                      RNDD      , 
                      RPDD      , 
                      KNDD      , 
                      KPDD      , 
                      RNDF      , 
                      RPDF      , 
                      VSSdeg    , 
                      VDDdeg    , 
                      RNUD      , 
                      RPUD      , 
                      KNUD      , 
                      KPUD      , 
                      RNUF      , 
                      RPUF      , 
                      QN        , 
                      KN        , 
                      QP        , 
                      KP        , 
                      KGUD      , 
                      KGDD      , 
                      INFB      , 
                      IPFB      ; 
extern info_list      TAS_INFO ;
extern double         TAS_CAPARAPREC ;
extern double         TAS_CAPASWITCH ;
extern double         TAS_CAPAOUT ;
extern char           *TAS_FILENAME ;
extern long           TAS_LIMITMEM ;
extern char           TAS_TRACE_MODE ;
extern char           TAS_PERFILE ;
extern char           TAS_FACTORISE ;
extern char           TAS_FIND_MIN ;
extern char           TAS_NB_VERSION[] ;
extern char           TAS_DATE_VER[] ;
extern char           TAS_SLOFILE ;
extern char           TAS_PERFINT ;
extern char           TAS_INT_END ;
extern char           TAS_NOTAS ;
extern char           TAS_DIF_LATCH ;
extern char           TAS_LANG ;
extern char           TAS_PWL ;
extern char           TAS_CNS_FILE ;
extern char           TAS_CNS_LOAD ;
extern char           TAS_TREATPRECH ;
extern char           TAS_SILENTMODE ;
extern char           TAS_INF ;
extern char           TAS_HIER ;
extern char           TAS_NHIER ;
extern char           TAS_CALCRCN ;
extern char           TAS_SAVERCN ;


/* declaration de fonction */

extern void           tas_initcnsfigalloc() ;
extern void           tas_freecnsfigalloc() ;
extern void           tas_freecnsfig() ;
extern void           tas_freelofig() ;
extern int            tas_timing() ;
extern int            tas_drislo() ;
extern int            tas_error() ;
extern ttvfig_list   *tas_cns2ttv() ;
extern int            tas_capara() ;
extern int            tas_DetectConeConnect() ;
extern long           tas_prelink() ;
extern int            tas_para() ;
extern int            tas_switch_cmos() ;
extern void           tas_AddCapaOut() ;
extern void           tas_AddCapaDiff() ;
extern int            tas_StuckAt() ;
extern list_list     *tas_GetAliasSig() ;
extern int            tas_DifLatch() ;
extern int            tas_TechnoParameters() ;
extern int            tas_tpd() ;
extern int            tas_PwlFile() ;
extern int            tas_setenv() ;
extern int            tas_option() ;
extern int            tas_CloseTerm() ;
extern int            tas_RestoreTerm() ;
extern void           tas_GetKill() ;
extern info_list      tas_InitInfo() ;
extern int            tas_PrintTime() ;
extern void           tas_visu() ;
extern ttvfig_list   *tas_main() ;

#endif
